"""
base.py is modified from sqlacodegen.py, which is generated by sqlacodegen.

Notes:
    1. Add BaseModel class to make use of AllFeaturesMixin, don't forget to import it.
    2. Each model needs to inherit BaseModel rather than Base.

You can compare differences between base.py and sqlacodegen.py before modification.

"""

# coding: utf-8
from sqlalchemy import BigInteger, Boolean, CHAR, Column, ForeignKey, Index, JSON, SmallInteger, String, Table, Text, UniqueConstraint, text
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base

from sqlalchemy_mixins import AllFeaturesMixin

Base = declarative_base()
metadata = Base.metadata


class BaseModel(Base, AllFeaturesMixin):
    __abstract__ = True


class Addressbook(BaseModel):
    __tablename__ = 'addressbook'

    id = Column(BigInteger, primary_key=True, server_default=text("nextval('addressbook_id_seq'::regclass)"))
    uri = Column(String(255), nullable=False, unique=True, comment='use uuid4')
    display_name = Column(String(255), nullable=False, index=True)
    color = Column(String(255), nullable=False)
    description = Column(String(255))
    type = Column(SmallInteger, server_default=text("1"), comment='0: public\n1: normal\n2: suggested\n3: default\n4: public_default')
    created_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))
    modified_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))


class ExternalSource(BaseModel):
    __tablename__ = 'external_source'

    id_addressbook = Column(ForeignKey('addressbook.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True)
    source = Column(String(32), nullable=False, server_default=text("''::character varying"))
    username = Column(String(1024), nullable=False, server_default=text("''::character varying"))
    token = Column(Text, nullable=False, server_default=text("''::text"))
    addressbook_location = Column(String(2048), nullable=False, server_default=text("''::character varying"))
    status = Column(SmallInteger, nullable=False, server_default=text("0"))
    modified_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))


class AddressbookObjectKind(BaseModel):
    __tablename__ = 'addressbook_object_kind'

    kind = Column(SmallInteger, primary_key=True)
    description = Column(Text)


t_addressbook_object_metadata_without_photo_vcard_view = Table(
    'addressbook_object_metadata_without_photo_vcard_view', metadata,
    Column('id', BigInteger),
    Column('id_addressbook', BigInteger),
    Column('uri', String(255)),
    Column('kind', SmallInteger),
    Column('md5', CHAR(32)),
    Column('is_hidden', Boolean),
    Column('usage_frequency', BigInteger),
    Column('created_time', BigInteger),
    Column('modified_time', BigInteger),
    Column('full_name', String(1024)),
    Column('company', String(1024)),
    Column('department', String(1024)),
    Column('job_title', String(1024)),
    Column('email', String(1024)),
    Column('telephone', String(1024)),
    Column('date', String(1024)),
    Column('birthday', String(1024))
)


class AddressbookObjectSearchTokenType(BaseModel):
    __tablename__ = 'addressbook_object_search_token_type'

    type = Column(SmallInteger, primary_key=True)
    description = Column(Text)


t_addressbook_object_with_metadata_view = Table(
    'addressbook_object_with_metadata_view', metadata,
    Column('id', BigInteger),
    Column('uri', String(255)),
    Column('vcard_text', Text),
    Column('kind', SmallInteger),
    Column('md5', CHAR(32)),
    Column('is_hidden', Boolean),
    Column('created_time', BigInteger),
    Column('modified_time', BigInteger),
    Column('id_addressbook', BigInteger),
    Column('usage_frequency', BigInteger),
    Column('photo', Text),
    Column('display_name', String(1024)),
    Column('company', String(1024)),
    Column('department', String(1024)),
    Column('job_title', String(1024)),
    Column('email', String(1024)),
    Column('telephone', String(1024)),
    Column('date', String(1024)),
    Column('birthday', String(1024))
)


class AddressbookType(BaseModel):
    __tablename__ = 'addressbook_type'

    type = Column(SmallInteger, primary_key=True)
    description = Column(Text)


class Config(BaseModel):
    __tablename__ = 'config'

    key = Column(String(255), primary_key=True)
    value = Column(Text, nullable=False)


class DirectoryObject(BaseModel):
    __tablename__ = 'directory_object'

    id = Column(BigInteger, primary_key=True)
    json_data = Column(JSON, server_default=text("'{}'::json"))


class DirectoryObjectCustomSetting(BaseModel):
    __tablename__ = 'directory_object_custom_setting'

    json_data = Column(JSON, server_default=text("'{}'::json"))
    id_directory_object = Column(ForeignKey('directory_object.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, unique=True)


class LabelType(BaseModel):
    __tablename__ = 'label_type'

    type = Column(SmallInteger, primary_key=True)
    description = Column(Text)


t_mailclient_migration_type = Table(
    'mailclient_migration_type', metadata,
    Column('type', SmallInteger),
    Column('description', Text)
)


class ManyPrincipalHasManyAddressbookMode(BaseModel):
    __tablename__ = 'many_principal_has_many_addressbook_mode'

    mode = Column(SmallInteger, primary_key=True)
    description = Column(Text)


class ManyPrincipalHasManyAddressbookStatu(BaseModel):
    __tablename__ = 'many_principal_has_many_addressbook_status'

    status = Column(SmallInteger, primary_key=True)
    description = Column(Text)


class OrganizationUnit(BaseModel):
    __tablename__ = 'organization_unit'

    id = Column(BigInteger, primary_key=True, server_default=text("nextval('organization_unit_id_seq'::regclass)"))
    dn = Column(Text, nullable=False)
    created_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))
    modified_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))

    principal = relationship('Principal', secondary='organization_unit_has_many_principal')


class Principal(BaseModel):
    __tablename__ = 'principal'
    __table_args__ = (
        UniqueConstraint('id', 'owner_id'),
    )

    id = Column(BigInteger, primary_key=True, server_default=text("nextval('principal_id_seq'::regclass)"))
    owner_id = Column(BigInteger, nullable=False, index=True, comment='Based on principal type:\n system principals:\n   -1: Everyone\n user/group:\n    uid/gid')
    type = Column(SmallInteger, nullable=False, comment='1: user\n2: group')
    display_name = Column(String(1024), comment='SYNO_USER_FULLNAME_UTF8_MAX: 64*6+1')
    account_name = Column(String(1024), comment='SYNO_USERNAME_UTF8_MAX: 492')
    status = Column(SmallInteger, server_default=text("0"))
    created_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))
    disabled_time = Column(BigInteger, server_default=text("0"), comment='if a user/group is removed in dsm, the corresponding principal is set to disabled instead of being deleted')
    modified_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))

    parents = relationship(
        'Principal',
        secondary='principal_many_group_has_many_member',
        primaryjoin='Principal.id == principal_many_group_has_many_member.c.id_group',
        secondaryjoin='Principal.id == principal_many_group_has_many_member.c.id_member'
    )


t_principal_id_to_addressbook_privilege_view = Table(
    'principal_id_to_addressbook_privilege_view', metadata,
    Column('id_principal', BigInteger),
    Column('owner_id_principal', BigInteger),
    Column('id_addressbook', BigInteger),
    Column('mode', SmallInteger),
    Column('status', SmallInteger)
)


t_principal_id_to_addressbook_view = Table(
    'principal_id_to_addressbook_view', metadata,
    Column('id_principal', BigInteger),
    Column('owner_id_principal', BigInteger),
    Column('id', BigInteger),
    Column('uri', String(255)),
    Column('display_name', String(255)),
    Column('color', String(255)),
    Column('description', String(255)),
    Column('type', SmallInteger),
    Column('created_time', BigInteger),
    Column('modified_time', BigInteger),
    Column('mode', SmallInteger),
    Column('status', SmallInteger)
)


class PrincipalStatu(BaseModel):
    __tablename__ = 'principal_status'

    status = Column(SmallInteger, primary_key=True)
    description = Column(Text)


class PrincipalType(BaseModel):
    __tablename__ = 'principal_type'

    type = Column(SmallInteger, primary_key=True)
    description = Column(Text)


t_principal_with_group_id_view = Table(
    'principal_with_group_id_view', metadata,
    Column('id', BigInteger),
    Column('owner_id', BigInteger),
    Column('type', SmallInteger),
    Column('display_name', String(1024)),
    Column('account_name', String(1024)),
    Column('status', SmallInteger),
    Column('created_time', BigInteger),
    Column('disabled_time', BigInteger),
    Column('modified_time', BigInteger),
    Column('id_group', BigInteger)
)


class AddressbookObject(BaseModel):
    __tablename__ = 'addressbook_object'

    id = Column(BigInteger, primary_key=True, server_default=text("nextval('addressbook_object_id_seq'::regclass)"))
    uri = Column(String(255), nullable=False, unique=True, comment='use uuid4')
    vcard_text = Column(Text, nullable=False)
    kind = Column(SmallInteger, nullable=False, comment='0: person\n1: group')
    md5 = Column(CHAR(32), nullable=False)
    is_hidden = Column(Boolean, server_default=text("false"))
    created_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))
    modified_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))
    id_addressbook = Column(ForeignKey('addressbook.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'))
    usage_frequency = Column(BigInteger, nullable=False, server_default=text("0"))

    addressbook = relationship('Addressbook')
    directory_object = relationship('DirectoryObject', secondary='id_directory_object_to_id_addressbook_object')
    label = relationship('Label', secondary='many_label_has_many_addressbook_object')
    parents = relationship(
        'AddressbookObject',
        secondary='addressbook_object_many_group_has_many_member',
        primaryjoin='AddressbookObject.id == addressbook_object_many_group_has_many_member.c.id_group',
        secondaryjoin='AddressbookObject.id == addressbook_object_many_group_has_many_member.c.id_member'
    )


class AddressbookObjectMetadata(BaseModel):
    __tablename__ = 'addressbook_object_metadata'

    id_addressbook_object = Column(ForeignKey('addressbook_object.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, unique=True)
    full_name = Column(String(1024))
    company = Column(String(1024))
    department = Column(String(1024))
    job_title = Column(String(1024))
    email = Column(String(1024))
    telephone = Column(String(1024))
    date = Column(String(1024))
    birthday = Column(String(1024))
    photo = Column(Text)


class AddressbookObjectRevision(BaseModel):
    __tablename__ = 'addressbook_object_revision'
    __table_args__ = (
        Index('addressbook_object_id_id_addressbook_idx', 'id', 'id_addressbook'),
    )

    id = Column(BigInteger, primary_key=True, server_default=text("nextval('addressbook_object_revision_id_seq'::regclass)"))
    id_principal = Column(ForeignKey('principal.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), nullable=False)
    id_addressbook = Column(ForeignKey('addressbook.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), nullable=False)
    uri = Column(String(255), nullable=False, comment='same as uri in addressbook_object')
    is_deleted = Column(Boolean, nullable=False)
    modified_time = Column(BigInteger, nullable=False, server_default=text("(date_part('epoch'::text, now()))::bigint"))

    addressbook = relationship('Addressbook')
    principal = relationship('Principal')


class Label(BaseModel):
    __tablename__ = 'label'
    __table_args__ = (
        UniqueConstraint('id', 'id_principal'),
    )

    id = Column(BigInteger, primary_key=True, server_default=text("nextval('label_id_seq'::regclass)"))
    id_principal = Column(ForeignKey('principal.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), index=True)
    display_name = Column(String(255), nullable=False)
    color = Column(String(255), nullable=False)
    type = Column(SmallInteger, server_default=text("0"), comment='0: normal\n1: starred')

    principal = relationship('Principal')


t_mailclient_migration = Table(
    'mailclient_migration', metadata,
    Column('id_principal', ForeignKey('principal.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL')),
    Column('id_contacts_pkg', BigInteger, nullable=False),
    Column('id_mailclient_pkg', BigInteger, nullable=False),
    Column('type', SmallInteger, nullable=False),
    UniqueConstraint('id_principal', 'id_contacts_pkg', 'id_mailclient_pkg', 'type')
)


class ManyPrincipalHasManyAddressbook(BaseModel):
    __tablename__ = 'many_principal_has_many_addressbook'

    id_principal = Column(ForeignKey('principal.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False)
    mode = Column(SmallInteger, nullable=False, comment='3: own\n2: manage\n1: write\n0: read')
    status = Column(SmallInteger, nullable=False, comment='2: accepted\n1: declined\n0: invited')
    id_addressbook = Column(ForeignKey('addressbook.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False, index=True)

    addressbook = relationship('Addressbook')
    principal = relationship('Principal')


t_organization_unit_has_many_principal = Table(
    'organization_unit_has_many_principal', metadata,
    Column('id_organization_unit', ForeignKey('organization_unit.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL')),
    Column('id_principal', ForeignKey('principal.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'))
)


t_principal_many_group_has_many_member = Table(
    'principal_many_group_has_many_member', metadata,
    Column('id_group', ForeignKey('principal.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False),
    Column('id_member', ForeignKey('principal.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False, index=True)
)


t_addressbook_object_many_group_has_many_member = Table(
    'addressbook_object_many_group_has_many_member', metadata,
    Column('id_group', ForeignKey('addressbook_object.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False),
    Column('id_member', ForeignKey('addressbook_object.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False)
)


t_addressbook_object_search_token = Table(
    'addressbook_object_search_token', metadata,
    Column('id_addressbook_object', ForeignKey('addressbook_object.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), index=True),
    Column('token', String(1024)),
    Column('type', SmallInteger, comment='0: given_name\n1: middle_name\n2: family_name\n3: job_title\n4: department\n5: company\n6: email\n7: telephone\n8: address\n9: url\n10: date\n11: instant_messenger\n12: phonetic_given_name\n13: phonetic_middle_name\n14: phonetic_family_name\n15: nickname')
)


t_id_directory_object_to_id_addressbook_object = Table(
    'id_directory_object_to_id_addressbook_object', metadata,
    Column('id_directory_object', ForeignKey('directory_object.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False),
    Column('id_addressbook_object', ForeignKey('addressbook_object.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False)
)


t_many_label_has_many_addressbook_object = Table(
    'many_label_has_many_addressbook_object', metadata,
    Column('id_label', ForeignKey('label.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False),
    Column('id_addressbook_object', ForeignKey('addressbook_object.id', ondelete='CASCADE', onupdate='CASCADE', match='FULL'), primary_key=True, nullable=False, index=True)
)
