from email.utils import formatdate
from radicale.storage import sanitize_path
from radicale.syno.lib import SynoLib


def get_sane_path(path):
    return sanitize_path(str(path)).strip("/")


def get_attrs(path):
    sane_path = get_sane_path(path)
    return sane_path.split("/")


def get_principal_uri_and_addressbook_uri(path):
    attrs = get_attrs(path)
    principal_uri = attrs[0] if len(attrs) > 0 else None
    addressbook_uri = attrs[1] if len(attrs) > 1 else None
    return principal_uri, addressbook_uri


def get_http_date(unix_time_second):
    return formatdate(
        timeval=int(unix_time_second),
        localtime=False,
        usegmt=True
    )


def set_uid_to_object(uid, vobject_item):
    """Set UID value of an addressbook object."""
    if vobject_item.name == "VCARD":
        vobject_item.uid.value = uid


def user_name_to_principal_uri(user_name):
    """
    Transfer the user_name to real name and then replace "\" by "@@"
    """
    user_real_name = SynoLib().get_real_name(user_name)
    return user_real_name.replace("\\", "@@")


def principal_uri_to_user_name(principal_uri):
    """
    Replace "@@" by "\"
    """
    return principal_uri.replace("@@", "\\")
